<?php

namespace Drupal\dynamic_library_loader\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration form for the Dynamic Library Loader module.
 */
class DynamicLibraryLoaderForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['dynamic_library_loader.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dynamic_library_loader_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('dynamic_library_loader.settings');

    // Initialize rows in the form state if not already set.
    if ($form_state->get('library_mappings') === NULL) {
      $form_state->set('library_mappings', $config->get('library_mappings') ?? []);
    }

    $entries = $form_state->get('library_mappings');

    // Add help text to guide administrators.
    $form['help'] = [
      '#type' => 'markup',
      '#markup' => $this->t('<p>Context examples:</p>
        <ul>
          <li><strong>Content types:</strong> <em>article</em></li>
          <li><strong>Taxonomy terms:</strong> <em>tags</em></li>
          <li><strong>Paragraphs:</strong> <em>image_gallery</em></li>
          <li><strong>Views:</strong> <em>my_view:block_1</em></li>
        </ul>
        <p>Select the appropriate context for each row.</p>'),
    ];

    // Wrap the table and button in a container for AJAX replacement.
    $form['library_mappings_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'dynamic-library-mappings-wrapper'],
    ];

    // Build the table structure.
    $form['library_mappings_wrapper']['library_mappings'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Context Type'),
        $this->t('Context (Machine Name)'),
        $this->t('Theme/Module'),
        $this->t('Library'),
        $this->t('Actions'),
      ],
      '#empty' => $this->t('No mappings available. Add a new row.'),
    ];

    // Populate existing rows.
    foreach ($entries as $index => $entry) {
      $form['library_mappings_wrapper']['library_mappings'][$index]['context_type'] = [
        '#type' => 'select',
        '#options' => [
          'content_type' => $this->t('Content Type'),
          'taxonomy_term' => $this->t('Taxonomy Term'),
          'paragraph' => $this->t('Paragraph'),
          'view' => $this->t('View'),
          //'block_type' => $this->t('Block Type'),
        ],
        '#default_value' => $entry['context_type'] ?? '',
      ];
      $form['library_mappings_wrapper']['library_mappings'][$index]['context'] = [
        '#type' => 'textfield',
        '#default_value' => $entry['context'] ?? '',
        '#placeholder' => $this->t('e.g., article, tags, my_view:block_1'),
      ];
      $form['library_mappings_wrapper']['library_mappings'][$index]['theme'] = [
        '#type' => 'textfield',
        '#default_value' => $entry['theme'] ?? '',
        '#placeholder' => $this->t('e.g., mytheme'),
      ];
      $form['library_mappings_wrapper']['library_mappings'][$index]['library'] = [
        '#type' => 'textfield',
        '#default_value' => $entry['library'] ?? '',
        '#placeholder' => $this->t('e.g., mylibrary'),
      ];
      $form['library_mappings_wrapper']['library_mappings'][$index]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#submit' => [[$this, 'removeRowSubmit']],
        '#name' => "remove_{$index}",
        '#attributes' => ['class' => ['button--small']],
        '#limit_validation_errors' => [],
        '#ajax' => [
          'callback' => [$this, 'ajaxCallback'],
          'wrapper' => 'dynamic-library-mappings-wrapper',
        ],
      ];
    }

    // Add a button to add new rows.
    $form['library_mappings_wrapper']['add_row'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Row'),
      '#submit' => [[$this, 'addRowSubmit']],
      '#ajax' => [
        'callback' => [$this, 'ajaxCallback'],
        'wrapper' => 'dynamic-library-mappings-wrapper',
      ],
    ];

    // Add the Save Configuration button outside of the AJAX container.
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save Configuration'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Submit handler for adding a row.
   */
  public function addRowSubmit(array &$form, FormStateInterface $form_state) {
    $library_mappings = $form_state->get('library_mappings') ?? [];
    $library_mappings[] = ['context_type' => '', 'context' => '', 'theme' => '', 'library' => '']; // Add an empty row.
    $form_state->set('library_mappings', $library_mappings);
    $form_state->setRebuild();
  }

  /**
   * Submit handler for removing a row.
   */
  public function removeRowSubmit(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $name_parts = explode('_', $triggering_element['#name']);
    $index = end($name_parts);

    $library_mappings = $form_state->get('library_mappings') ?? [];
    unset($library_mappings[$index]); // Remove the row.
    $form_state->set('library_mappings', $library_mappings);
    $form_state->setRebuild();
  }

  /**
   * AJAX callback to refresh the table and buttons.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    return $form['library_mappings_wrapper'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $library_mappings = $form_state->getValue('library_mappings') ?? [];

    // Ensure $library_mappings is an array
    if (!is_array($library_mappings)) {
      $library_mappings = []; // Initialize as an empty array
    }

    foreach ($library_mappings as $index => $entry) {
      // ... rest of your validation logic ...
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $library_mappings = $form_state->getValue('library_mappings') ?? [];

    // Ensure $library_mappings is an array
    if (!is_array($library_mappings)) {
      $library_mappings = [];
    }

    // Filter out empty rows
    $filtered_mappings = array_filter($library_mappings, function ($entry) {
      return !empty($entry['context_type']) && !empty($entry['context']) && !empty($entry['theme']) && !empty($entry['library']);
    });

    // Update the configuration with the filtered mappings
    $this->config('dynamic_library_loader.settings')
      ->set('library_mappings', $filtered_mappings)
      ->save();

    // **Preserve the form state for subsequent form submissions**
    $form_state->setValue('library_mappings', $filtered_mappings);
  }
}